@@ -5,12 +5,14 @@ from sklearn.naive_bayes import GaussianNB
 from sklearn.neural_network import MLPClassifier
 from sklearn.svm import LinearSVC
 from waterdetect.Common import DWConfig, DWutils
+from waterdetect import calinski_harabasz_score
 
 
 class DWImageClustering:
 
-    def __init__(self, bands, bands_keys, invalid_mask, config: DWConfig):
+    def __init__(self, bands, bands_keys, invalid_mask, glint_processor, config: DWConfig):
 
+        self.glint_processor = glint_processor
         self.config = config
         self.data_as_columns = None
         self.clusters_labels = None
@@ -132,7 +134,7 @@ class DWImageClustering:
             cluster_model = cluster.AgglomerativeClustering(n_clusters=self.best_k, linkage=self.config.linkage)
 
         cluster_model.fit(data)
-        return cluster_model.labels_
+        return cluster_model.labels_.astype('int8')
 
     def find_best_k(self, data):
         """
@@ -169,7 +171,7 @@ class DWImageClustering:
                 print('k={} :Silhouete index={}'.format(num_k, computed_metrics[num_k - min_k]))
 
             else:
-                computed_metrics.append(metrics.calinski_harabasz_score(data, labels))
+                computed_metrics.append(calinski_harabasz_score(data, labels))
                 print('k={} :Calinski_harabaz index={}'.format(num_k, computed_metrics[num_k - min_k]))
 
 
@@ -230,6 +232,8 @@ class DWImageClustering:
                 raise OSError('NDWI band necessary for detecting water with minmir2 option')
             water_cluster = self.detect_cluster('value', 'max', 'ndwi')
 
+        elif self.config.detect_water_cluster == 'minnir':
+            water_cluster = self.detect_cluster('value', 'min', 'Nir')
 
         else:
             raise OSError('Method {} for detecting water cluster does not exist'.
@@ -293,7 +297,7 @@ class DWImageClustering:
         else:
             clusters_labels = self.apply_naive_bayes(data, clusters_labels, train_data)
 
-        return clusters_labels
+        return clusters_labels.astype('int8')
 
     @staticmethod
     def apply_svm(data, clusters_labels, clusters_data):
@@ -388,7 +392,7 @@ class DWImageClustering:
         # todo: treat no_data value of -9999
 
         # create an empty matrix
-        matrice_cluster = np.zeros_like(list(self.bands.values())[0])
+        matrice_cluster = np.zeros_like(list(self.bands.values())[0]).astype('int8')
 
         # apply water pixels to value 1
         matrice_cluster[indices_array[0][self.clusters_labels == self.water_cluster['clusterid']],
@@ -578,7 +582,7 @@ class DWImageClustering:
             self.cluster_matrix = self.apply_clustering()
 
         # self.water_mask = self.cluster_matrix == 1
-        self.water_mask = np.where(self.cluster_matrix == 1, 1, np.where(self.invalid_mask == 1, 255, 0))
+        self.water_mask = np.where(self.cluster_matrix == 1, 1, np.where(self.invalid_mask == 1, 255, 0)).astype('int8')
 
         return self.cluster_matrix
 
@@ -642,16 +646,33 @@ class DWImageClustering:
             self.clusters_labels = train_clusters_labels
 
         # after obtaining the final labels, clip bands with superior limit
+
         for band, value in zip(self.config.clip_band, self.config.clip_sup_value):
             if value is not None:
+                if self.config.glint_mode:
+                    comp_array = self.glint_processor.glint_adjusted_threshold(band,
+                                                                               value,
+                                                                               'SUP',
+                                                                               self.invalid_mask)
+                else:
+                    comp_array = value
+
                 self.clusters_labels[(self.clusters_labels == self.water_cluster['clusterid']) &
-                                     (self.bands[band][~self.invalid_mask] > value)] = -1
+                                     (self.bands[band][~self.invalid_mask] > comp_array)] = -1
 
         # after obtaining the final labels, clip bands with inferior limit
         for band, value in zip(self.config.clip_band, self.config.clip_inf_value):
             if value is not None:
+                if self.config.glint_mode:
+                    comp_array = self.glint_processor.glint_adjusted_threshold(band,
+                                                                               value,
+                                                                               'INF',
+                                                                               self.invalid_mask)
+                else:
+                    comp_array = value
+
                 self.clusters_labels[(self.clusters_labels == self.water_cluster['clusterid']) &
-                                     (self.bands[band][~self.invalid_mask] < value)] = -1
+                                     (self.bands[band][~self.invalid_mask] < comp_array)] = -1
 
         # create an cluster array based on the cluster result (water will be value 1)
         return self.create_matrice_cluster(ind_data)
